\\ Read codes of sizes 7 and 8 in G(2,4).
Codes = readvec("G2_4_data.txt");

\r rtrip.gp
allocatemem(10^9);

\\ Compute inner product of matrices A and B.  Much better than
\\ rtrip_trace(rtrip_matrix_mult(A,B)).
rtrip_inner_product(A,B) = rtrip_vecsum(vector(matsize(A)[1],i,rtrip_dot_product(A[i,],B[i,])));

\\ Test whether a code in G(2,4) is a tight simplex.
test(C) = {
    N = length(C);
    for(i=1,N,for(j=1,4,for(k=1,4,  \\ Check isolating intervals
        if(!rtrip_verify(C[i][j,k]),return(0)))));
    for(i=1,N,  \\ Check symmetry
        if(!rtrip_matrix_equal(C[i],C[i]~),return(0)));
    for(i=1,N,  \\ Check trace
        if(!rtrip_equal(rtrip_trace(C[i]),rtrip_rational(2)),return(0)));
    for(i=1,N,  \\ Check idempotence
        if(!rtrip_matrix_equal(rtrip_matrix_mult(C[i],C[i]),C[i]),return(0)));
    for(i=1,N,for(j=i+1,N,  \\ Check distances
        if(!rtrip_equal(rtrip_inner_product(C[i],C[j]),rtrip_rational((N-2)/(N-1))),return(0))));
    return(1);
};

print("7 points in G(2,4): ",if(test(Codes[1]),"verified","failed"));
print("8 points in G(2,4): ",if(test(Codes[2]),"verified","failed"));
