\\ ----------------------------------------------------------------------------
\\ Extract the points. Note that this also sets N, n, m, and k, which are global
\\ variables used below, and sets eta to be 1.

get_point(data) = {
    N = #data;
    m = #(data[1]);
    n = #(data[1][1]);
    k = (N-1)/n;
    if(N!=11 || n!=5 || m!=2, error("Incorrect data"));  \\ Keep file from being used by accident.
    return(concat(flatten(vector(k,i,data[i])),[1]));
}
\\ ----------------------------------------------------------------------------

\\ ----------------------------------------------------------------------------
\\ Variables

cyclic_shift(v) = concat(vector(n-1,i,v[i+1]),v[1]);
\\ Point x_{i,j} (1 <= i <= N-1, 1 <= j <= m) with n coordinates
pt(i,j) = if(i<=k,vector(n,p,eval(Str("x",i,"c",j,"p",p))),cyclic_shift(pt(i-k,j)));

\\ List of all variables
all_variables() = { concat(flatten(vector(k,i,vector(m,j,pt(i,j)))),
                           [Eta]) };
\\ Eta is capitalized since eta is a function name in pari.
\\ ----------------------------------------------------------------------------

\\ ----------------------------------------------------------------------------
\\ Helpful definitions

proj_matrix(i) = sum(j=1,m,pt(i,j)~*pt(i,j));
mat_inner_prod(A,B) = trace(A*B);
\\ ----------------------------------------------------------------------------

\\ ----------------------------------------------------------------------------
\\ Constraints

all_constraints() = {
    local(constraints,Pi_N,M);
    constraints = [];
    \\ vector length constraints
    for(i=1,k,for(j=1,m,
        constraints=concat(constraints, [-1 + nm(pt(i,j))])));
    \\ orthogonality constraints
    for(i=1,k,for(j1=1,m,for(j2=j1+1,m,
        constraints=concat(constraints, [pt(i,j1)*pt(i,j2)~]))));
    \\ equidistance constraints
    for(i=1,k,for(q=1,floor(n/2),if([i,q] != [1,1],
        constraints=concat(constraints, [mat_inner_prod(proj_matrix(i),proj_matrix(i+q*k))-mat_inner_prod(proj_matrix(1),proj_matrix(k+1))]))));
    for(i=1,k-1,for(i0=i+1,k,for(q=0,n-1,
        constraints=concat(constraints, [mat_inner_prod(proj_matrix(i),proj_matrix(i0+q*k))-mat_inner_prod(proj_matrix(1),proj_matrix(k+1))]))));
    \\ N-th projection matrix
    Pi_N = N*m/n*matid(n) - sum(i=1,N-1,proj_matrix(i));
    M = Pi_N^2 - Eta*Pi_N;
    for(i=1,floor(n/2)+1,
        constraints=concat(constraints,[M[1,i]]));
    return(constraints);
}
\\ ----------------------------------------------------------------------------
