\\ ----------------------------------------------------------------------------
\\ Extract the points. Note that this also sets N, n, and m, which are global
\\ variables used below, and defines the w's to be n/(N*m).

get_point(data) = {
    N = #data;
    m = #(data[1]);
    n = #(data[1][1]);
    return(concat(flatten(data),
                  vector(N,X,n/(N*m))));
}
\\ ----------------------------------------------------------------------------

\\ ----------------------------------------------------------------------------
\\ Variables

\\ Point x_{i,j} (1 <= i <= N, 1 <= j <= m) with n coordinates
pt(i,j) = { vector(n,k,eval(Str("x",i,"c",j,"p",k))) };

\\ List of all variables
all_variables() = { concat(flatten(vector(N,i,vector(m,j,pt(i,j)))),
                           vector(N,i,eval(Str("w",i)))) };
\\ ----------------------------------------------------------------------------

\\ ----------------------------------------------------------------------------
\\ Helpful definitions

proj_matrix(i) = sum(j=1,m,pt(i,j)~*pt(i,j));
mat_inner_prod(A,B) = trace(A*B);
weighted_sum_proj(j1,j2) = sum(i=1,N,eval(Str("w",i))*proj_matrix(i))[j1,j2];
\\ ----------------------------------------------------------------------------

\\ ----------------------------------------------------------------------------
\\ Constraints

all_constraints() = {
    local(constraints);
    constraints = [];
    \\ vector length constraints
    for(i=1,N,for(j=1,m,
        constraints=concat(constraints, [-1 + nm(pt(i,j))])));
    \\ orthogonality constraints
    for(i=1,N,for(j1=1,m,for(j2=j1+1,m,
        constraints=concat(constraints, [pt(i,j1)*pt(i,j2)~]))));
    \\ equidistance constraints
    for(i=1,N,for(j=i+1,N,if([i,j]!=[1,2],
        constraints=concat(constraints, [mat_inner_prod(proj_matrix(i),proj_matrix(j))-mat_inner_prod(proj_matrix(1),proj_matrix(2))]))));
    \\ diagonal sum of projections
    for(i=1,n,
        constraints=concat(constraints, [-1 + weighted_sum_proj(i,i)]));
    \\ off-diagonal sum of projections
    for(i=1,n,for(j=i+1,n,
        constraints=concat(constraints, [weighted_sum_proj(i,j)])));
    return(constraints);
}
\\ ----------------------------------------------------------------------------
