\\ ----------------------------------------------------------------------------
\\ Extract the points. Note that this also sets N and d, which are global
\\ variables used below.

get_point(data) = {
    local(points);
    N = #data;
    d = #(data[1]);
    if(N!=13 || d!=3, error("Incorrect data"));  \\ Keep file from being used by accident.
    m = (N-1)/d;
    points = vector(m,i,data[i]);  \\ Only pay attention to first m points in data.
    return(flatten(points));
}
\\ ----------------------------------------------------------------------------

\\ ----------------------------------------------------------------------------
\\ Variables

\\ Point i, component j (incorporating cyclic symmetry)
pt(i,j) = { if(i<=m,vector(4,k,eval(Str("x",i,"c",j,"p",k))),
                    pt(i-m,(j%d)+1)) };

\\ List of all variables
all_variables() = { flatten(vector(m,i,vector(d,j,pt(i,j)))) };
\\ ----------------------------------------------------------------------------

\\ ----------------------------------------------------------------------------
\\ Helpful definitions

point_length(i) = sum(j=1,d,nm(pt(i,j)));
cos_sq(i1,i2) = nm(sum(j=1,d,mul(cnj(pt(i1,j)), pt(i2,j))));
proj_matrix(i,j1,j2) = mul(pt(i,j1), cnj(pt(i,j2)));
sum_proj(j1,j2) = sum(i=1,N-1,proj_matrix(i,j1,j2));
\\ ----------------------------------------------------------------------------

\\ ----------------------------------------------------------------------------
\\ Constraints

all_constraints() = {
    local(constraints);
    constraints = [];
    \\ vector length constraints
    for(i=1,m,
        constraints=concat(constraints, [-1 + point_length(i)]));
    \\ equidistance constraints
    for(i=1,m,for(j=i+1,N,if([i,j]!=[1,2],
        if((j == i+m) || (j <= m) || ((j > i+m) && (j <= 2*m)) || ((j > i+2*m) && (j <= 3*m)),
            constraints=concat(constraints, [cos_sq(i,j) - cos_sq(1,2)])))));
    \\ sum of projections
    constraints=concat(constraints, [sum_proj(1,2)[1] - 1/6]);
    constraints=concat(constraints, [nm(sum_proj(1,2)) - 1/9]);
    return(constraints);
}
\\ ----------------------------------------------------------------------------
