\\ ----------------------------------------------------------------------------
\\ Extract the points. Note that this also sets N and d, which are global
\\ variables used below, and initializes variables for |x_i|^4 to be 1.

get_point(data) = {
    N = #data;
    d = #(data[1]);
    if(N!=15 || d!=3, error("Incorrect data"));  \\ Keep file from being used by accident.
    return(concat(flatten(data),
                  vector(N,i,1)));
}
\\ ----------------------------------------------------------------------------

\\ ----------------------------------------------------------------------------
\\ Variables

\\ Point i, component j
pt(i,j) = { vector(4,k,eval(Str("x",i,"c",j,"p",k))) };

\\ |x_i|^4
normfourth_variable(i) = eval(Str("normfourth",i));

\\ List of all variables
all_variables() = { concat(flatten(vector(N,i,vector(d,j,pt(i,j)))),
                           vector(N,i,normfourth_variable(i))) };
\\ ----------------------------------------------------------------------------

\\ ----------------------------------------------------------------------------
\\ Helpful definitions

point_length(i) = sum(j=1,d,nm(pt(i,j)));
proj_matrix(i,j1,j2) = mul(pt(i,j1), cnj(pt(i,j2)));
gamma_matrix(i,j1,j2) = proj_matrix(i,j1,j2) - [if(j1==j2,point_length(i)/3,0),0,0,0];
gamma_dot(i1,i2) = sum(i=1,d,sum(j=1,d,mul(cnj(gamma_matrix(i1,i,j)),gamma_matrix(i2,i,j))[1]));
\\ ----------------------------------------------------------------------------

\\ ----------------------------------------------------------------------------
\\ Constraints

all_constraints() = {
    local(constraints);
    constraints = [];
    \\ |x_i|^4 should be normfouth
    for(i=1,N,
        constraints=concat(constraints, [point_length(i)^2 - normfourth_variable(i)]));
    \\ dot product constrsints
    for(i=1,N,for(j=i+1,N,
        constraints=concat(constraints, [gamma_dot(i,j)+1/21])));
    return(constraints);
}
\\ ----------------------------------------------------------------------------
