\\ ----------------------------------------------------------------------------
\\ Extract the points. Note that this also sets N and d, which are global
\\ variables used below, and defines the w's to be d/N.

get_point(data) = {
    N = #data;
    d = #(data[1]);
    return(concat(flatten(data),
                  vector(N,i,d/N)));
}
\\ ----------------------------------------------------------------------------

\\ ----------------------------------------------------------------------------
\\ Variables

\\ Point i, component j (arbitrary variable names for pari)
pt(i,j) = { vector(4,k,eval(Str("x",i,"c",j,"p",k))) };

\\ List of all variables
all_variables() = { concat(flatten(vector(N,i,vector(d,j,pt(i,j)))),
                           vector(N,i,eval(Str("w",i)))) };
\\ ----------------------------------------------------------------------------

\\ ----------------------------------------------------------------------------
\\ Helpful definitions

point_length(i) = sum(j=1,d,nm(pt(i,j)));
cos_sq(i1,i2) = nm(sum(j=1,d,mul(cnj(pt(i1,j)), pt(i2,j))));
proj_matrix(i,j1,j2) = mul(pt(i,j1), cnj(pt(i,j2)));
weighted_sum_proj(j1,j2) = sum(i=1,N,eval(Str("w",i))*proj_matrix(i,j1,j2));
\\ ----------------------------------------------------------------------------

\\ ----------------------------------------------------------------------------
\\ Constraints

all_constraints() = {
    local(constraints);
    constraints = [];
    \\ vector length constraints
    for(i=1,N,
        constraints=concat(constraints, [-1 + point_length(i)]));
    \\ equidistance constraints
    for(i=1,N,for(j=i+1,N,if([i,j]!=[1,2],
        constraints=concat(constraints, [cos_sq(i,j) - cos_sq(1,2)]))));
    \\ diagonal sum of projections (note entries are real)
    for(i=1,d,
        constraints=concat(constraints, [-1 + weighted_sum_proj(i,i)[1]]));
    \\ off-diagonal sum of projections
    for(i=1,d,for(j=i+1,d,for(k=1,4,
        constraints=concat(constraints, [weighted_sum_proj(i,j)[k]]))));
    return(constraints);
}
\\ ----------------------------------------------------------------------------
