\\ ----------------------------------------------------------------------------
\\ Extract the points. Note that this also sets N and d, which are global
\\ variables used below, and defines the w's to be 1/m.

get_point(data) = {
    local(points);
    points = data;
    N = #points;
    d = #(points[1]);
    if(N!=24 || d!=3, error("Incorrect data"));  \\ Keep file from being used by accident.
    m = N/d;
    points = vector(m,i,points[i]);  \\ Only pay attention to first m points in data.
    for(i=1,m,points[i][1]=[points[i][1][1]]);  \\ first coordinate is real
    return(concat(flatten(points),
                  vector(m,i,1/m)));
}
\\ ----------------------------------------------------------------------------

\\ ----------------------------------------------------------------------------
\\ Variables

\\ Point i, component j (incorporating cyclic symmetry)
pt_(i,j)     = { if(j==1,
                   [eval(Str("x",i,"c1p1")),0,0,0,0,0,0,0],
                   vector(8,k,eval(Str("x",i,"c",j,"p",k)))) };
pt(i,j) = if(i<=m,pt_(i,j),pt(i-m,(j%d)+1));

\\ Only need this up to m
ptvars(i,j) = { if(j==1,
                  [eval(Str("x",i,"c1p1"))],
                  vector(8,k,eval(Str("x",i,"c",j,"p",k)))) };

\\ Auxiliary variable
w_variable(i) = if(i<=m,eval(Str("w",i)),w_variable(i-m));

\\ List of all variables
all_variables() = { concat(flatten(vector(m,i,vector(d,j,ptvars(i,j)))),
                           vector(m,i,w_variable(i))) };
\\ ----------------------------------------------------------------------------

\\ ----------------------------------------------------------------------------
\\ Helpful definitions

point_length(i) = sum(j=1,d,nm(pt(i,j)));
proj_matrix(i,j1,j2) = mul(pt(i,j1), cnj(pt(i,j2)));
cos_sq(i1,i2) = sum(i=1,d,sum(j=1,d,mul(cnj(proj_matrix(i1,i,j)),proj_matrix(i2,i,j))[1]));
weighted_sum_proj(j1,j2) = sum(i=1,N,w_variable(i)*proj_matrix(i,j1,j2));
\\ ----------------------------------------------------------------------------

\\ ----------------------------------------------------------------------------
\\ Constraints

all_constraints() = {
    local(constraints);
    constraints = [];
    \\ vector length constraints
    for(i=1,m,
        constraints=concat(constraints, [-1 + point_length(i)]));
    \\ equidistance constraints
    for(i=1,m,for(j=i+1,N,if([i,j]!=[1,2],
        if((j == i+m) || (j <= m) || ((j > i+m) && (j <= 2*m)) || ((j > i+2*m) && (j <= 3*m)),
            constraints=concat(constraints, [cos_sq(i,j) - cos_sq(1,2)])))));
    \\ sum of projections (note 1,1 entry is real)
    constraints=concat(constraints, [-1 + weighted_sum_proj(1,1)[1]]);
    for(k=1,8,
        constraints=concat(constraints, [weighted_sum_proj(1,2)[k]]));
    return(constraints);
}
\\ ----------------------------------------------------------------------------
