\\ ----------------------------------------------------------------------------
\\ Extract the points. Note that this also sets N and d, which are global
\\ variables used below, and defines the w's to be d/N.

get_point(data) = {
    local(points);
    N = #data;
    d = #(data[1]);
    if(d!=3,error("Incorrect dimension"));
    points=data;
    for(i=1,N,points[i][1]=[points[i][1][1]]); \\ first coordinate is real
    return(concat(flatten(points),
                  vector(N,i,d/N)));
}
\\ ----------------------------------------------------------------------------

\\ ----------------------------------------------------------------------------
\\ Variables

\\ Point i, component j
pt(i,j)     = { if(j==1,
                  [eval(Str("x",i,"c1p1")),0,0,0,0,0,0,0],
                  vector(8,k,eval(Str("x",i,"c",j,"p",k)))) };
ptvars(i,j) = { if(j==1,
                  [eval(Str("x",i,"c1p1"))],
                  vector(8,k,eval(Str("x",i,"c",j,"p",k)))) };

\\ List of all variables
all_variables() = { concat(flatten(vector(N,i,vector(d,j,ptvars(i,j)))),
                           vector(N,i,eval(Str("w",i)))) };
\\ ----------------------------------------------------------------------------

\\ ----------------------------------------------------------------------------
\\ Helpful definitions

point_length(i) = sum(j=1,d,nm(pt(i,j)));
proj_matrix(i,j1,j2) = mul(pt(i,j1), cnj(pt(i,j2)));
cos_sq(i1,i2) = sum(i=1,d,sum(j=1,d,mul(cnj(proj_matrix(i1,i,j)),proj_matrix(i2,i,j))[1]));
weighted_sum_proj(j1,j2) = sum(i=1,N,eval(Str("w",i))*proj_matrix(i,j1,j2));
\\ ----------------------------------------------------------------------------

\\ ----------------------------------------------------------------------------
\\ Constraints

all_constraints() = {
    local(constraints);
    constraints = [];
    \\ vector length constraints
    for(i=1,N,
        constraints=concat(constraints, [-1 + point_length(i)]));
    \\ equidistance constraints
    for(i=1,N,for(j=i+1,N,if([i,j]!=[1,2],
        constraints=concat(constraints, [cos_sq(i,j) - cos_sq(1,2)]))));
    \\ diagonal sum of projections (note entries are real)
    for(i=1,d,
        constraints=concat(constraints, [-1 + weighted_sum_proj(i,i)[1]]));
    \\ off-diagonal sum of projections
    for(i=1,d,for(j=i+1,d,for(k=1,8,
        constraints=concat(constraints, [weighted_sum_proj(i,j)[k]]))));
    return(constraints);
}
\\ ----------------------------------------------------------------------------
