\\ ----------------------------------------------------------------------------
\\ Proof wrapper

\\ Call this with the name of a setup file, e.g., 'hp_general.gp', and
\\ the required data (the list of points).
\\
\\ The global variable number_succeeded is used to keep track of whether
\\ every proof has succeeded.

number_succeeded = 0;

do_proof(setupfile, data, {eps=10^(-9), denom=2^50}) = {
    local(point, variables, constraints, result);

    read(setupfile);
    point = get_point(data);
    variables = all_variables();
    constraints = all_constraints();

    \\ run the proof
    result = check_a_proof(variables, constraints, point, eps, 0, denom);
    \\ replace with the following to check how good jacobian_inverse needs to be:
    \\ result = (proof_bound(variables, constraints, point, eps) > 10^(-2));

    if(result==1, number_succeeded = number_succeeded + 1);
    return(result);
};


\\ ----------------------------------------------------------------------------

\r rigorous_proof.gp
allocatemem(10^9);

projective_cases = readvec("projective_data.txt");
grass_cases = readvec("grass_data.txt");

{
for(i=1,#projective_cases,
    result = if(do_proof(projective_cases[i][2],projective_cases[i][3]),"verified","failed");
    print(projective_cases[i][1],": ",result));
}

{
for(i=1,#grass_cases,
    result = if(do_proof(grass_cases[i][2],grass_cases[i][3]),"verified","failed");
    print(grass_cases[i][1],": ",result));
}

{
if(number_succeeded < #projective_cases + #grass_cases,
    print("A proof failed or ran out of memory."),
    print("Every proof was verified."));
}

\\ To check individual cases interactively, you can comment out the for loops above
\\ and use the following function, with a descriptive string input.  For example,
\\ check_a_case("11 points in G(2,5)") or check_a_case("27 points in OP^2").
\\ The string must be formatted exactly this way or it will return nothing.

{
check_a_case(s) = 
    for(i=1,length(projective_cases),
        if(projective_cases[i][1]==s,return(do_proof(projective_cases[i][2],projective_cases[i][3]))));
    for(i=1,length(grass_cases),
        if(grass_cases[i][1]==s,return(do_proof(grass_cases[i][2],grass_cases[i][3]))));
}
